// Copyright 2019 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package storage

import (
	"context"
	"testing"
)

func TestMem(t *testing.T) {
	TestStorage(t, context.Background(), new(Mem))
}
